document.addEventListener('DOMContentLoaded', function() {
    const templateHtml = `
        <div class="meta-field">
            <input type="text" name="bw_variation_meta_fields[labels][]" value="" placeholder="Field Label" class="field-label">
            <input type="text" name="bw_variation_meta_fields[descriptions][]" value="" placeholder="Field Description">
            <label class="hide-field-label">
                <input type="checkbox" name="bw_variation_meta_fields[hide][]" value="">
                Hide on product
            </label>
            <button type="button" class="button remove-field">Remove</button>
        </div>
    `;

    // Fonction pour convertir le label en slug
    function convertToSlug(text) {
        return text
            .toLowerCase()
            .replace(/[^a-z0-9]+/g, '-')
            .replace(/(^-|-$)/g, '');
    }

    document.getElementById('add-field').addEventListener('click', function() {
        document.getElementById('meta-fields-container').insertAdjacentHTML('beforeend', templateHtml);
        
        // Ajouter l'écouteur d'événements au nouveau champ
        const newField = document.getElementById('meta-fields-container').lastElementChild;
        const labelInput = newField.querySelector('.field-label');
        const hideCheckbox = newField.querySelector('[name="bw_variation_meta_fields[hide][]"]');
        
        labelInput.addEventListener('input', function() {
            const slug = convertToSlug(this.value);
            hideCheckbox.value = slug;
        });
    });

    // Event delegation pour les boutons de suppression
    document.getElementById('meta-fields-container').addEventListener('click', function(e) {
        if (e.target && e.target.classList.contains('remove-field')) {
            e.target.closest('.meta-field').remove();
        }
    });

    // Event delegation pour les changements de label
    document.getElementById('meta-fields-container').addEventListener('input', function(e) {
        if (e.target && e.target.classList.contains('field-label')) {
            const fieldContainer = e.target.closest('.meta-field');
            const hideCheckbox = fieldContainer.querySelector('[name="bw_variation_meta_fields[hide][]"]');
            const slug = convertToSlug(e.target.value);
            hideCheckbox.value = slug;
        }
    });

    // Initialiser les valeurs pour les champs existants
    document.querySelectorAll('.field-label').forEach(function(labelInput) {
        const fieldContainer = labelInput.closest('.meta-field');
        const hideCheckbox = fieldContainer.querySelector('[name="bw_variation_meta_fields[hide][]"]');
        const slug = convertToSlug(labelInput.value);
        hideCheckbox.value = slug;
    });
});