<?php defined( 'ABSPATH' ) || exit;
/**
 * @author  Bruno Pirard
 * @package BW
 * @since   1.0.0
 * @version 0.1.0
 *
 * @wordpress-plugin
 * Plugin Name:     WooCommerce Custom Variation Meta
 * Description:     Add custom fields to variations.
 * Version:         0.1.0
 * Plugin URI:      https://bulgaweb.com/plugins
 * Author:          Bruno Pirard
 * Author URI:      https://bulgaweb.com/
 * Text Domain:     woocommerce-custom-variation-meta
 * Domain Path:     /languages/
 * 
 * Requires PHP:    7.4
 * WC requires at least: 6.0
 * WC tested up to: 8.2
 * Requires at least: 5.0
 * WC HPOS Compatible: yes
 * 
 * License:         GPL-2.0 or later
 * License URI:     http://www.gnu.org/licenses/gpl-2.0.html
 *
 * @copyright 2017-2024 Bruno Pirard
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * ( at your option ) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

define('BW_VARIATION_HPOS_COMPATIBLE', true);

define('BW_VARIATION_VERSION', '0.1.0');

define('BW_VARIATION_URL', plugin_dir_url(__FILE__));

define('BW_VARIATION_PATH', plugin_dir_path(__FILE__));

define('BW_VARIATION_ASSETS_URL', BW_VARIATION_URL . 'assets/');


require_once BW_VARIATION_PATH . 'includes/class-bw-variation-settings.php';
$bw_variation_settings = BW_Variation_Settings::get_instance();
//require_once plugin_dir_path(__FILE__) . 'includes/uninstall.php';

$version_woo = '6.0';

// HPOS compatibility
add_action('before_woocommerce_init', function() {
    if (class_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class)) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
    }
});

// Add action to enqueue admin scripts and styles
add_action('admin_enqueue_scripts', 'bw_variation_admin_scripts');

// Add action to enqueue frontend scripts and styles
add_action('wp_enqueue_scripts', 'bw_variation_frontend_scripts');

// Load plugin text domain
add_action('plugins_loaded', 'bw_load_variation_textdomain');

add_action( 'woocommerce_product_after_variable_attributes', 'bw_add_variation_options_other_dimensions', 10, 3 );

add_action( 'woocommerce_admin_process_variation_object', 'bw_save_variation_options_other_dimensions', 10 ,2 );

add_filter( 'woocommerce_available_variation', 'bw_available_variation', 10, 3 );

add_action( 'woocommerce_single_variation', 'bw_add_variation_dimension_holder', 15 );

add_action('woocommerce_before_single_variation', 'bw_add_variation_dimension_holder', 10);

add_action( 'woocommerce_after_single_product_summary', 'bw_variable_footer_scripts', 15 );

add_filter( 'woocommerce_locate_template', 'woo_adon_plugin_template', 1, 3 );

add_action('wp_footer', 'bw_variable_footer_scripts');

add_action('woocommerce_save_product_variation', 'bw_save_variation_data', 10, 2);

//register_uninstall_hook(__FILE__, 'mon_plugin_uninstall');



/**
 * Check if WooCommerce is active
 *
 * @return boolean
 */
function bulga_variation_is_woocommerce_active() {
    if (!defined('WC_VERSION')) {       
        // Deactivate the plugin
        deactivate_plugins(plugin_basename(BULGA_PICKUP_PLUGIN_FILE));
        
        // Die with error
        wp_die(sprintf(
            // translators: %s: WooCommerce URL 
            __('This plugin requires WooCommerce to be installed and activated. You can download %s here. <br><a href="%s">Back to plugins page</a>', 'bulga-pickup'),
            '<a href="https://wordpress.org/plugins/woocommerce/" target="_blank">WooCommerce</a>',
            admin_url('plugins.php')
        ));
    } 
    else {
        if (version_compare( WC_VERSION, $version_woo, '<' )) {
            // Deactivate the plugin
            deactivate_plugins(plugin_basename(BULGA_PICKUP_PLUGIN_FILE));
        
            // Die with error
            wp_die(sprintf(
                // translators: %s: WooCommerce URL 
                __('This plugin requires a version of WooCommerce higher than 8.0. You can download %s here. <br><a href="%s">Back to plugins page</a>', 'bulga-pickup'),
                '<a href="https://wordpress.org/plugins/woocommerce/" target="_blank">WooCommerce</a>',
                admin_url('plugins.php')
            ));
        } 
    }
    return true;
}

/**
 * Enqueue admin scripts and styles for the variation meta fields settings page.
 *
 * Hooks into 'admin_enqueue_scripts' action.
 *
 * @since 1.0.0
 *
 * @param string $hook The current admin page hook.
 */
function bw_variation_admin_scripts($hook) {
    // Only load on our settings page
    if ('woocommerce_page_bw-variation-meta-settings' !== $hook) {
        return;
    }
    
    wp_enqueue_style(
        'bw-variation-admin-styles',
        BW_VARIATION_ASSETS_URL . 'css/bw-variation-styles.css',
        array(),
        BW_VARIATION_VERSION
    );
    
    wp_enqueue_script(
        'bw-variation-frontend',
        BW_VARIATION_ASSETS_URL . 'js/bw-variation-frontend.js',
        array(), // Removed jQuery dependency
        BW_VARIATION_VERSION,
        true
    );
}

/**
 * Enqueue frontend scripts and styles for the variation dimensions output.
 *
 * Hooks into 'wp_enqueue_scripts' action.
 *
 * @since 1.0.0
 */
function bw_variation_frontend_scripts() {
    if (!is_product()) {
        return;
    }
    
    wp_enqueue_style(
        'bw-variation-styles',
        BW_VARIATION_ASSETS_URL . 'css/bw-variation-styles.css',
        array(),
        BW_VARIATION_VERSION
    );
    
    wp_enqueue_script(
        'bw-variation-frontend',
        BW_VARIATION_ASSETS_URL . 'js/bw-variation-frontend.js',
        array('jquery'), // Ajoutez jQuery comme dépendance
        BW_VARIATION_VERSION,
        true
    );
}

/**
 * Loads the plugin text domain for localization.
 *
 * Uses the `load_plugin_textdomain` function to load the
 * `woocommerce-custom-variation-meta` text domain. The second argument is the path
 * to the `languages` directory, which is located in the plugin directory.
 *
 * @since 1.0.0
 */
function bw_load_variation_textdomain() {
    load_plugin_textdomain( 'woocommerce-custom-variation-meta', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
}

/**
 * Add custom fields to product variation settings
 *
 * @param string $loop
 * @param array $variation_data
 * @param WP_Post $variation
 * @return array
 */
function bw_add_variation_options_other_dimensions($loop, $variation_data, $variation) {
    $variation_obj = wc_get_product($variation->ID);
    $unit = get_option('woocommerce_dimension_unit');
    
    if (class_exists('Automattic\WooCommerce\Utilities\I18nUtil')) {
        $unit = Automattic\WooCommerce\Utilities\I18nUtil::get_dimensions_unit_label($unit);
    }

    $settings = BW_Variation_Settings::get_instance();
    $meta_fields = $settings->get_meta_fields();

    if (!empty($meta_fields['fields'])) {
        $counter = 0;
        foreach ($meta_fields['fields'] as $key => $field) {
            woocommerce_wp_text_input(array(
                'id' => $field . '[' . $loop . ']',
                'class' => 'short',
                'label' => sprintf(esc_html__('%s (%s)', 'woocommerce-custom-variation-meta'), 
                    $meta_fields['labels'][$key], 
                    esc_html($unit)
                ),
                'desc_tip' => 'true',
                'description' => $meta_fields['descriptions'][$key],
                'value' => $variation_obj->get_meta('_' . $field, true),
                'placeholder' => wc_format_localized_decimal(0),
                'wrapper_class' => 'form-row ' . ($counter % 2 == 0 ? 'form-row-first' : 'form-row-last'),
            ));
            $counter++;
        }
    }
}

/**
 * Save custom fields values for product variation
 *
 * @param WC_Product_Variation $variation
 * @param int $i
 * @return void
 */
function bw_save_variation_options_other_dimensions($variation, $i) {
    $settings = BW_Variation_Settings::get_instance();
    $meta_fields = $settings->get_meta_fields();

    if (!empty($meta_fields['labels'])) {
        foreach ($meta_fields['labels'] as $label) {
            $field = sanitize_title($label);
            if (isset($_POST[$field][$i])) {
                $variation->update_meta_data('_' . $field, wc_clean($_POST[$field][$i]));
            }
        }
    }
}

/**
 * Save custom field values for product variation.
 *
 * @param int $variation_id - Post ID of the product variation.
 * @param int $i - Index of the variation.
 *
 * @return void
 */
function bw_save_variation_data($variation_id, $i) {
    $settings = BW_Variation_Settings::get_instance();
    $meta_fields = $settings->get_meta_fields();

    if (!empty($meta_fields['fields'])) {
        foreach ($meta_fields['fields'] as $field) {
            $field_name = $field . '[' . $i . ']';
            if (isset($_POST[$field_name])) {
                update_post_meta($variation_id, '_' . $field, sanitize_text_field($_POST[$field_name]));
            }
        }
    }
}

/**
 * Modifies the available variation data to include custom meta fields.
 *
 * This function retrieves the custom variation meta fields from settings and 
 * appends them to the available variation data if they are not marked to be hidden.
 * The data is formatted as HTML and includes a unit if applicable.
 *
 * @param array $data The variation data array to be modified.
 * @param WC_Product $product The product object to which the variation belongs.
 * @param WC_Product_Variation $variation The variation object to be processed.
 *
 * @return array The modified variation data with custom fields included.
 */
function bw_available_variation($data, $product, $variation) {
    $settings = BW_Variation_Settings::get_instance();
    $meta_fields = $settings->get_meta_fields();
    $hide_fields = isset($meta_fields['hide']) ? $meta_fields['hide'] : array();
    $unit = get_option('woocommerce_dimension_unit');
    
    if (!empty($meta_fields['labels'])) {
        foreach ($meta_fields['labels'] as $key => $label) {
            $field = sanitize_title($label); // Générer l'ID à partir du label
            
            // Skip if field is in hide array
            if (in_array($field, $hide_fields)) {
                continue;
            }

            $value = $variation->get_meta('_' . $field);
            if ($value) {
                $data[$field . '_html'] = sprintf(
                    '<div class="variation-custom-field"><span class="label">%s:</span> <span class="value">%s %s</span></div>',
                    esc_html($label),
                    esc_html($value),
                    esc_html($unit)
                );
            }
        }
    }

    return $data;
}

/**
 * Utility to format dimension for display.
 *
 * @param  string $dimension_string Single dimension.
 * @return string
 */
function bw_format_single_dimensions( $dimension_string ) {

	if ( ! empty( $dimension_string ) ) {

        $dimension_label = get_option( 'woocommerce_dimension_unit' );

        if ( class_exists( 'Automattic\WooCommerce\Utilities\I18nUtil' )  ) {
          $dimension_label = Automattic\WooCommerce\Utilities\I18nUtil::get_dimensions_unit_label( $dimension_label );
        }

		$dimension_string = sprintf(
			// translators: 1. A formatted number; 2. A label for a dimensions unit of measure. E.g. 3.14 cm.
			esc_html_x( '%1$s %2$s', 'formatted dimensions', 'bulga-variations-meta' ),
			$dimension_string,
			$dimension_label
		);
	} else {
		$dimension_string = esc_html__( 'N/A', 'bulga-variations-meta' );
	}

	return $dimension_string;
}

/**
 * Add a <div> element to hold dimension output.
 * The class name needs to match the find() statement in bw_variable_footer_scripts()
 */
function bw_add_variation_dimension_holder() {
    ?>
    <div class="woocommerce-variation-custom-data"></div>
    <?php
}

/**
 * Filters the path to a single template part.
 *
 * This allows the modification of the template part path to include the
 * variation template generated by the plugin.
 *
 * @param string $template      The path to the template part.
 * @param string $template_name The name of the template part.
 * @param string $template_path The path to the template part.
 *
 * @return string The modified path to the template part.
 */
function woo_adon_plugin_template($template, $template_name, $template_path) {
    global $woocommerce;
    
    if ($template_name !== 'single-product/add-to-cart/variation.php') {
        return $template;
    }

    // Générer le contenu du template
    $template_content = bw_generate_variation_template();
    
    // Créer un fichier temporaire avec le contenu
    $temp_file = tempnam(sys_get_temp_dir(), 'wc_variation_');
    file_put_contents($temp_file, $template_content);
    
    return $temp_file;
}

/**
 * Generates the JavaScript template for the WooCommerce variation display.
 *
 * This function generates the template based on the custom fields defined in the admin settings page.
 * It checks if each field should be hidden or not, and outputs the corresponding HTML.
 *
 * @since 1.0.0
 * @return string The generated template HTML.
 */
function bw_generate_variation_template() {
    $settings = BW_Variation_Settings::get_instance();
    $meta_fields = $settings->get_meta_fields();
    
    ob_start();
    ?>
    <script type="text/template" id="tmpl-variation-template">
        <div class="woocommerce-variation-description">{{{ data.variation.variation_description }}}</div>
        <div class="woocommerce-variation-price">{{{ data.variation.price_html }}}</div>
        <?php
        if (!empty($meta_fields['fields'])) {
            foreach ($meta_fields['fields'] as $key => $field) {
                // Vérifier si le champ doit être caché
                if (!isset($meta_fields['hide']) || !in_array($field, $meta_fields['hide'])) {
                    ?>
                    <div class="woocommerce-variation-<?php echo esc_attr($field); ?>">{{{ data.variation.<?php echo esc_attr($field); ?>_html }}}</div>
                    <?php
                }
            }
        }
        ?>
    </script>
    <script type="text/template" id="tmpl-unavailable-variation-template">
        <p role="alert"><?php esc_html_e('Sorry, this product is unavailable. Please choose a different combination.', 'woocommerce'); ?></p>
    </script>
    <?php
    return ob_get_clean();
}

/**
 * Outputs JavaScript templates for WooCommerce product variations.
 *
 * This function checks if the current page is a product page and if the product is of type variable. 
 * It retrieves custom variation meta fields defined in the admin settings and outputs JavaScript 
 * templates used to display variation-specific information on the frontend, including descriptions, 
 * price, availability, and custom data. Fields marked as hidden are excluded from the output.
 *
 * Hooks into 'wp_footer' action.
 *
 * @since 1.0.0
 */
function bw_variable_footer_scripts() {
    if (!is_product()) {
        return;
    }

    global $product;
    if (!$product || !$product->is_type('variable')) {
        return;
    }

    $settings = BW_Variation_Settings::get_instance();
    $meta_fields = $settings->get_meta_fields();
    
    if (empty($meta_fields['fields'])) {
        return;
    }

    ?>
    <script type="text/template" id="tmpl-variation-template">
        <div class="woocommerce-variation-description">{{{ data.variation.variation_description }}}</div>
        <div class="woocommerce-variation-price">{{{ data.variation.price_html }}}</div>
        <div class="woocommerce-variation-availability">{{{ data.variation.availability_html }}}</div>
        <div class="woocommerce-variation-custom-data">
            <?php
            foreach ($meta_fields['fields'] as $key => $field) {
                if (!isset($meta_fields['hide']) || !in_array($field, $meta_fields['hide'])) {
                    echo '<div class="woocommerce-variation-' . esc_attr($field) . '">{{{ data.variation.' . esc_attr($field) . '_html }}}</div>';
                }
            }
            ?>
        </div>
    </script>
    <script type="text/template" id="tmpl-unavailable-variation-template">
        <p><?php esc_html_e('Sorry, this product is unavailable. Please choose a different combination.', 'woocommerce'); ?></p>
    </script>
    <?php
}